@@ -5,14 +5,48 @@
 #include <stdint.h>
 #include <time.h>
 #include <math.h>
+#include <zip.h>
+
+/* Windows has no MIN/MAX macros - so let's just roll our own */
+#define MIN(x, y) ({                \
+    typeof(x) _min1 = (x);          \
+    typeof(y) _min2 = (y);          \
+    (void) (&_min1 == &_min2);      \
+    _min1 < _min2 ? _min1 : _min2; })
+
+#define MAX(x, y) ({                \
+    typeof(x) _max1 = (x);          \
+    typeof(y) _max2 = (y);          \
+    (void) (&_max1 == &_max2);      \
+    _max1 > _max2 ? _max1 : _max2; })
 
-#include <glib.h>
-#include <glib/gstdio.h>
 #include <libxml/tree.h>
 #include <libxslt/transform.h>
 
 #include "sha1.h"
 
+#ifdef __cplusplus
+extern "C" {
+#else
+#if __STDC_VERSION__ >= 199901L
+#include <stdbool.h>
+#ifndef TRUE
+#define TRUE true
+#endif
+#ifndef FALSE
+#define FALSE false
+#endif
+#else
+typedef int bool;
+#ifndef TRUE
+#define TRUE 1
+#endif
+#ifndef FALSE
+#define FALSE 0
+#endif
+#endif
+#endif
+
 #define O2_IN_AIR		209     // permille
 #define N2_IN_AIR		781
 #define O2_DENSITY		1429    // mg/Liter
@@ -26,34 +60,6 @@
 #define SEAWATER_SALINITY	10300
 #define FRESHWATER_SALINITY	10000
 
-/* Dive tag definitions */
-#define DTAG_INVALID		(1 << 0)
-#define DTAG_BOAT		(1 << 1)
-#define DTAG_SHORE		(1 << 2)
-#define DTAG_DRIFT		(1 << 3)
-#define DTAG_DEEP		(1 << 4)
-#define DTAG_CAVERN		(1 << 5)
-#define DTAG_ICE		(1 << 6)
-#define DTAG_WRECK		(1 << 7)
-#define DTAG_CAVE		(1 << 8)
-#define DTAG_ALTITUDE		(1 << 9)
-#define DTAG_POOL		(1 << 10)
-#define DTAG_LAKE		(1 << 11)
-#define DTAG_RIVER		(1 << 12)
-#define DTAG_NIGHT		(1 << 13)
-#define DTAG_FRESH		(1 << 14)
-#define DTAG_FRESH_NR		14
-#define DTAG_STUDENT		(1 << 15)
-#define DTAG_INSTRUCTOR		(1 << 16)
-#define DTAG_PHOTO		(1 << 17)
-#define DTAG_VIDEO		(1 << 18)
-#define DTAG_DECO		(1 << 19)
-#define DTAG_NR			20
-/* defined in statistics.c */
-extern char *dtag_names[DTAG_NR];
-extern int dtag_shown[DTAG_NR];
-extern int dive_mask;
-
 /*
  * Some silly typedefs to make our units very explicit.
  *
@@ -89,7 +95,7 @@ extern int dive_mask;
  * We don't actually use these all yet, so maybe they'll change, but
  * I made a number of types as guidelines.
  */
-typedef gint64 timestamp_t;
+typedef int64_t timestamp_t;
 
 typedef struct {
 	int seconds;
@@ -138,6 +144,7 @@ typedef struct {
 	cylinder_type_t type;
 	struct gasmix gasmix;
 	pressure_t start, end, sample_start, sample_end;
+	depth_t depth;
 } cylinder_t;
 
 typedef struct {
@@ -145,16 +152,14 @@ typedef struct {
 	const char *description;	/* "integrated", "belt", "ankle" */
 } weightsystem_t;
 
-extern gboolean cylinder_nodata(cylinder_t *cyl);
-extern gboolean cylinder_none(void *_data);
-extern gboolean no_weightsystems(weightsystem_t *ws);
-extern gboolean weightsystems_equal(weightsystem_t *ws1, weightsystem_t *ws2);
-
 extern int get_pressure_units(unsigned int mb, const char **units);
 extern double get_depth_units(unsigned int mm, int *frac, const char **units);
 extern double get_volume_units(unsigned int ml, int *frac, const char **units);
 extern double get_temp_units(unsigned int mk, const char **units);
 extern double get_weight_units(unsigned int grams, int *frac, const char **units);
+extern double get_vertical_speed_units(unsigned int mms, int *frac, const char **units);
+
+extern unsigned int units_to_depth(double depth);
 
 static inline double grams_to_lbs(int grams)
 {
@@ -216,7 +221,7 @@ static inline double psi_to_bar(double psi)
 	return psi / 14.5037738;
 }
 
-static inline unsigned long psi_to_mbar(double psi)
+static inline long psi_to_mbar(double psi)
 {
 	return psi_to_bar(psi)*1000 + 0.5;
 }
@@ -233,6 +238,7 @@ static inline double bar_to_atm(double bar)
 
 /* Volume in mliter of a cylinder at pressure 'p' */
 extern int gas_volume(cylinder_t *cyl, pressure_t p);
+extern int wet_volume(double cuft, pressure_t p);
 
 static inline int mbar_to_PSI(int mbar)
 {
@@ -250,7 +256,7 @@ static inline int get_he(const struct gasmix *mix)
 	return mix->he.permille;
 }
 
-static inline gboolean is_air(int o2, int he)
+static inline bool is_air(int o2, int he)
 {
 	return (he == 0) && (o2 == 0 || ((o2 >= O2_IN_AIR - 1) && (o2 <= O2_IN_AIR + 1)));
 }
@@ -272,11 +278,50 @@ struct sample {
 	duration_t ndl;
 	duration_t stoptime;
 	depth_t stopdepth;
-	gboolean in_deco;
+	bool in_deco;
 	int cns;
 	int po2;
 };
 
+struct divetag {
+	/*
+	 * The name of the divetag. If a translation is available, name contains
+	 * the translated tag
+	 */
+	char *name;
+	/*
+	 * If a translation is available, we write the original tag to source.
+	 * This enables us to write a non-localized tag to the xml file.
+	 */
+	char *source;
+};
+
+struct tag_entry {
+	struct divetag *tag;
+	struct tag_entry *next;
+};
+
+/*
+ * divetags are only stored once, each dive only contains
+ * a list of tag_entries which then point to the divetags
+ * in the global g_tag_list
+ */
+
+extern struct tag_entry *g_tag_list;
+
+struct divetag *taglist_add_tag(struct tag_entry *tag_list, const char *tag);
+
+/*
+ * Writes all divetags in tag_list to buffer, limited by the buffer's (len)gth.
+ * Returns the characters written
+ */
+int taglist_get_tagstring(struct tag_entry *tag_list, char *buffer, int len);
+
+void taglist_init(struct tag_entry **tag_list);
+void taglist_clear(struct tag_entry *tag_list);
+void taglist_init_global();
+
+
 /*
  * Events are currently pretty meaningless. This is
  * just based on the random data that libdivecomputer
@@ -289,7 +334,7 @@ struct event {
 	struct event *next;
 	duration_t time;
 	int type, flags, value;
-	gboolean deleted;
+	bool deleted;
 	char name[];
 };
 
@@ -322,7 +367,7 @@ struct divecomputer {
 };
 
 #define MAX_CYLINDERS (8)
-#define MAX_WEIGHTSYSTEMS (4)
+#define MAX_WEIGHTSYSTEMS (6)
 #define W_IDX_PRIMARY 0
 #define W_IDX_SECONDARY 1
 
@@ -348,7 +393,7 @@ struct dive {
 	dive_trip_t *divetrip;
 	struct dive *next, **pprev;
 	int selected;
-	gboolean downloaded;
+	bool downloaded;
 	timestamp_t when;
 	char *location;
 	char *notes;
@@ -367,7 +412,7 @@ struct dive {
 	pressure_t surface_pressure;
 	duration_t duration;
 	int salinity; // kg per 10000 l
-        int dive_tags;
+	struct tag_entry *tag_list;
 
 	struct divecomputer dc;
 };
@@ -385,7 +430,7 @@ static inline void copy_gps_location(struct dive *from, struct dive *to)
 	}
 }
 
-static inline int get_surface_pressure_in_mbar(const struct dive *dive, gboolean non_null)
+static inline int get_surface_pressure_in_mbar(const struct dive *dive, bool non_null)
 {
 	int mbar = dive->surface_pressure.mbar;
 	if (!mbar && non_null)
@@ -468,6 +513,7 @@ struct units {
 	enum { BAR, PSI, PASCAL } pressure;
 	enum { CELSIUS, FAHRENHEIT, KELVIN } temperature;
 	enum { KG, LBS } weight;
+	enum { SECONDS, MINUTES } vertical_speed_time;
 };
 
 /*
@@ -482,7 +528,8 @@ struct units {
 	.volume = LITER,		\
 	.pressure = BAR,		\
 	.temperature = CELSIUS,		\
-	.weight = KG			\
+	.weight = KG,			\
+	.vertical_speed_time = MINUTES \
 }
 
 #define IMPERIAL_UNITS {		\
@@ -490,13 +537,14 @@ struct units {
 	.volume = CUFT,			\
 	.pressure = PSI,		\
 	.temperature = FAHRENHEIT,	\
-	.weight = LBS			\
+	.weight = LBS,			\
+	.vertical_speed_time = MINUTES \
 }
 extern const struct units SI_units, IMPERIAL_units;
 extern struct units xml_parsing_units;
 
 extern struct units *get_units(void);
-extern int verbose;
+extern int verbose, quit;
 
 struct dive_table {
 	int nr, allocated, preexisting;
@@ -549,7 +597,7 @@ static inline struct divecomputer *get_dive_dc(struct dive *dive, int nr)
 #define for_each_gps_location(_i,_x) \
 	for ((_i) = 0; ((_x) = get_gps_location(_i, &gps_location_table)) != NULL; (_i)++)
 
-static inline struct dive *get_dive_by_diveid(int diveid, int deviceid)
+static inline struct dive *get_dive_by_diveid(uint32_t diveid, uint32_t deviceid)
 {
 	int i;
 	struct dive *dive;
@@ -564,38 +612,34 @@ static inline struct dive *get_dive_by_diveid(int diveid, int deviceid)
 	return NULL;
 }
 extern struct dive *find_dive_including(timestamp_t when);
-extern gboolean dive_within_time_range(struct dive *dive, timestamp_t when, timestamp_t offset);
+extern bool dive_within_time_range(struct dive *dive, timestamp_t when, timestamp_t offset);
 struct dive *find_dive_n_near(timestamp_t when, int n, timestamp_t offset);
 
 /* Check if two dive computer entries are the exact same dive (-1=no/0=maybe/1=yes) */
 extern int match_one_dc(struct divecomputer *a, struct divecomputer *b);
 
 extern void parse_xml_init(void);
-extern void parse_xml_buffer(const char *url, const char *buf, int size, struct dive_table *table, GError **error);
+extern void parse_xml_buffer(const char *url, const char *buf, int size, struct dive_table *table, const char **params, char **error);
 extern void parse_xml_exit(void);
-extern void set_filename(const char *filename, gboolean force);
+extern void set_filename(const char *filename, bool force);
 
-extern int parse_dm4_buffer(const char *url, const char *buf, int size, struct dive_table *table, GError **error);
+extern int parse_dm4_buffer(const char *url, const char *buf, int size, struct dive_table *table, char **error);
 
-extern void parse_file(const char *filename, GError **error);
+extern void parse_file(const char *filename, char **error);
+extern void parse_csv_file(const char *filename, int time, int depth, int temp, int po2f, int cnsf, int stopdepthf, int sepidx, char **error);
 
-extern void show_dive_info(struct dive *);
-
-extern void show_dive_equipment(struct dive *, int w_idx);
-extern void clear_equipment_widgets(void);
-
-extern void show_dive_stats(struct dive *);
-extern void clear_stats_widgets(void);
-
-extern void show_gps_locations(void);
-extern void show_gps_location(struct dive *, void (*callback)(float, float));
-
-extern void show_yearly_stats(void);
-
-extern void update_dive(struct dive *new_dive);
 extern void save_dives(const char *filename);
-extern void save_dives_logic(const char *filename, gboolean select_only);
+extern void save_dives_logic(const char *filename, bool select_only);
 extern void save_dive(FILE *f, struct dive *dive);
+extern void export_dives_uddf(const char *filename, const bool selected);
+
+extern int subsurface_open(const char *path, int oflags, mode_t mode);
+extern FILE *subsurface_fopen(const char *path, const char *mode);
+extern void *subsurface_opendir(const char *path);
+extern struct zip *subsurface_zip_open_readonly(const char *path, int flags, int *errorp);
+extern int subsurface_zip_close(struct zip *zip);
+
+extern void shift_times(const timestamp_t amount);
 
 extern xsltStylesheetPtr get_stylesheet(const char *name);
 
@@ -609,44 +653,38 @@ extern struct sample *prepare_sample(struct divecomputer *dc);
 extern void finish_sample(struct divecomputer *dc);
 
 extern void sort_table(struct dive_table *table);
-extern void report_dives(gboolean imported, gboolean prefer_imported);
 extern struct dive *fixup_dive(struct dive *dive);
 extern unsigned int dc_airtemp(struct divecomputer *dc);
-extern struct dive *merge_dives(struct dive *a, struct dive *b, int offset, gboolean prefer_downloaded);
-extern struct dive *try_to_merge(struct dive *a, struct dive *b, gboolean prefer_downloaded);
+extern unsigned int dc_watertemp(struct divecomputer *dc);
+extern struct dive *merge_dives(struct dive *a, struct dive *b, int offset, bool prefer_downloaded);
+extern struct dive *try_to_merge(struct dive *a, struct dive *b, bool prefer_downloaded);
 extern void renumber_dives(int nr);
+extern void copy_events(struct dive *s, struct dive *d);
+extern void copy_cylinders(struct dive *s, struct dive *d);
+extern void copy_samples(struct dive *s, struct dive *d);
 
+extern void fill_default_cylinder(cylinder_t *cyl);
 extern void add_gas_switch_event(struct dive *dive, struct divecomputer *dc, int time, int idx);
 extern void add_event(struct divecomputer *dc, int time, int type, int flags, int value, const char *name);
+extern void per_cylinder_mean_depth(struct dive *dive, struct divecomputer *dc, int *mean, int *duration);
+extern int get_cylinder_index(struct dive *dive, struct event *ev);
 
 /* UI related protopypes */
 
-extern void init_ui(int *argcp, char ***argvp);
-
-extern void run_ui(void);
-extern void exit_ui(void);
-
-extern void report_error(GError* error);
+// extern void report_error(GError* error);
 
 extern void add_cylinder_description(cylinder_type_t *);
 extern void add_weightsystem_description(weightsystem_t *);
-extern void add_people(const char *string);
-extern void add_location(const char *string);
-extern void add_suit(const char *string);
 extern void remember_event(const char *eventname);
-extern int evn_foreach(void (*callback)(const char *, int *, void *), void *data);
+
+#if WE_DONT_USE_THIS /* this is a missing feature in Qt - selecting which events to display */
+extern int evn_foreach(void (*callback)(const char *, bool *, void *), void *data);
+#endif /* WE_DONT_USE_THIS */
+
 extern void clear_events(void);
 
-extern int add_new_dive(struct dive *dive);
-extern gboolean edit_trip(dive_trip_t *trip);
-extern int edit_dive_info(struct dive *dive, gboolean newdive);
-extern int edit_multi_dive_info(struct dive *single_dive);
-extern void dive_list_update_dives(void);
-extern void flush_divelist(struct dive *dive);
-
 extern void set_dc_nickname(struct dive *dive);
-extern const char *get_dc_nickname(const char *model, uint32_t deviceid);
-extern void set_autogroup(gboolean value);
+extern void set_autogroup(bool value);
 extern int total_weight(struct dive *);
 
 #define DIVE_ERROR_PARSE 1
@@ -656,6 +694,10 @@ const char *weekday(int wday);
 const char *monthname(int mon);
 
 #define UTF8_DEGREE "\xc2\xb0"
+#define UTF8_DELTA "\xce\x94"
+#define UTF8_UPWARDS_ARROW "\xE2\x86\x91"
+#define UTF8_DOWNWARDS_ARROW "\xE2\x86\x93"
+#define UTF8_AVERAGE "\xc3\xb8"
 #define UCS4_DEGREE 0xb0
 #define UTF8_SUBSCRIPT_2 "\xe2\x82\x82"
 #define UTF8_WHITESTAR "\xe2\x98\x86"
@@ -668,35 +710,27 @@ const char *monthname(int mon);
 #define FIVE_STARS	UTF8_BLACKSTAR UTF8_BLACKSTAR UTF8_BLACKSTAR UTF8_BLACKSTAR UTF8_BLACKSTAR
 extern const char *star_strings[];
 
-/* enum holding list of OS features */
-typedef enum {
-	UTF8_FONT_WITH_STARS
-} os_feature_t;
-
 extern const char *existing_filename;
-extern const char *subsurface_gettext_domainpath(char *);
-extern gboolean subsurface_os_feature_available(os_feature_t);
-extern gboolean subsurface_launch_for_uri(const char *);
-extern void subsurface_command_line_init(gint *, gchar ***);
-extern void subsurface_command_line_exit(gint *, gchar ***);
+extern void subsurface_command_line_init(int *, char ***);
+extern void subsurface_command_line_exit(int *, char ***);
 
 #define FRACTION(n,x) ((unsigned)(n)/(x)),((unsigned)(n)%(x))
 
 extern double add_segment(double pressure, const struct gasmix *gasmix, int period_in_seconds, int setpoint, const struct dive *dive);
 extern void clear_deco(double surface_pressure);
 extern void dump_tissues(void);
-extern unsigned int deco_allowed_depth(double tissues_tolerance, double surface_pressure, struct dive *dive, gboolean smooth);
-extern void set_gf(double gflow, double gfhigh);
+extern unsigned int deco_allowed_depth(double tissues_tolerance, double surface_pressure, struct dive *dive, bool smooth);
+extern void set_gf(short gflow, short gfhigh, bool gf_low_at_maxdepth);
 extern void cache_deco_state(double, char **datap);
 extern double restore_deco_state(char *data);
 
 struct divedatapoint {
 	int time;
-	int depth;
+	unsigned int depth;
 	int o2;
 	int he;
 	int po2;
-	gboolean entered;
+	bool entered;
 	struct divedatapoint *next;
 };
 
@@ -706,22 +740,71 @@ struct diveplan {
 	int surface_pressure;		/* mbar */
 	int bottomsac;			/* ml/min */
 	int decosac;			/* ml/min */
+	short gflow;
+	short gfhigh;
 	struct divedatapoint *dp;
 };
 
-void plan_add_segment(struct diveplan *diveplan, int duration, int depth, int o2, int he, int po2);
-void add_depth_to_nth_dp(struct diveplan *diveplan, int idx, int depth);
-void add_gas_to_nth_dp(struct diveplan *diveplan, int idx, int o2, int he);
-void free_dps(struct divedatapoint *dp);
+struct divedatapoint *plan_add_segment(struct diveplan *diveplan, int duration, int depth, int o2, int he, int po2);
 void get_gas_string(int o2, int he, char *buf, int len);
+struct divedatapoint *create_dp(int time_incr, int depth, int o2, int he, int po2);
+void dump_plan(struct diveplan *diveplan);
+void plan(struct diveplan *diveplan, char **cached_datap, struct dive **divep, bool add_deco, const char **error_string_p);
+void delete_single_dive(int idx);
 
 struct event *get_next_event(struct event *event, char *name);
 
-#ifdef DEBUGFILE
-extern char *debugfilename;
-extern FILE *debugfile;
+
+/* these structs holds the information that
+ * describes the cylinders / weight systems.
+ * they are global variables initialized in equipment.c
+ * used to fill the combobox in the add/edit cylinder
+ * dialog
+ */
+
+struct tank_info_t {
+	const char *name;
+	int cuft, ml, psi, bar;
+};
+extern struct tank_info_t tank_info[100];
+
+struct ws_info_t {
+	const char *name;
+	int grams;
+};
+extern struct ws_info_t ws_info[100];
+
+extern bool cylinder_nodata(cylinder_t *cyl);
+extern bool cylinder_none(void *_data);
+extern bool weightsystem_none(void *_data);
+extern bool no_weightsystems(weightsystem_t *ws);
+extern bool weightsystems_equal(weightsystem_t *ws1, weightsystem_t *ws2);
+extern void remove_cylinder(struct dive *dive, int idx);
+extern void remove_weightsystem(struct dive *dive, int idx);
+
+/*
+ * String handling.
+ */
+#define STRTOD_NO_SIGN		0x01
+#define STRTOD_NO_DOT		0x02
+#define STRTOD_NO_COMMA		0x04
+#define STRTOD_NO_EXPONENT	0x08
+extern double strtod_flags(const char *str, const char **ptr, unsigned int flags);
+
+#define STRTOD_ASCII (STRTOD_NO_COMMA)
+
+#define ascii_strtod(str,ptr) strtod_flags(str,ptr,STRTOD_ASCII)
+
+#ifdef __cplusplus
+}
 #endif
 
+extern weight_t string_to_weight(const char *str);
+extern depth_t string_to_depth(const char *str);
+extern pressure_t string_to_pressure(const char *str);
+extern volume_t string_to_volume(const char *str, pressure_t workp);
+extern fraction_t string_to_fraction(const char *str);
+
 #include "pref.h"
 
 #endif /* DIVE_H */
