@@ -4,8 +4,9 @@
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
-#include <glib/gi18n.h>
+#include "gettext.h"
 #include <zip.h>
+#include <time.h>
 
 #include "dive.h"
 #include "file.h"
@@ -24,7 +25,7 @@ int readfile(const char *filename, struct memblock *mem)
 	mem->buffer = NULL;
 	mem->size = 0;
 
-	fd = g_open(filename, O_RDONLY | O_BINARY, 0);
+	fd = subsurface_open(filename, O_RDONLY | O_BINARY, 0);
 	if (fd < 0)
 		return fd;
 	ret = fstat(fd, &st);
@@ -61,7 +62,7 @@ out:
 }
 
 
-static void zip_read(struct zip_file *file, GError **error, const char *filename)
+static void zip_read(struct zip_file *file, char **error, const char *filename)
 {
 	int size = 1024, n, read = 0;
 	char *mem = malloc(size);
@@ -72,15 +73,15 @@ static void zip_read(struct zip_file *file, GError **error, const char *filename
 		mem = realloc(mem, size);
 	}
 	mem[read] = 0;
-	parse_xml_buffer(filename, mem, read, &dive_table, error);
+	parse_xml_buffer(filename, mem, read, &dive_table, NULL, error);
 	free(mem);
 }
 
-static int try_to_open_zip(const char *filename, struct memblock *mem, GError **error)
+static int try_to_open_zip(const char *filename, struct memblock *mem, char **error)
 {
 	int success = 0;
 	/* Grr. libzip needs to re-open the file, it can't take a buffer */
-	struct zip *zip = zip_open(filename, ZIP_CHECKCONS, NULL);
+	struct zip *zip = subsurface_zip_open_readonly(filename, ZIP_CHECKCONS, NULL);
 
 	if (zip) {
 		int index;
@@ -92,12 +93,46 @@ static int try_to_open_zip(const char *filename, struct memblock *mem, GError **
 			zip_fclose(file);
 			success++;
 		}
-		zip_close(zip);
+		subsurface_zip_close(zip);
 	}
 	return success;
 }
 
-static int try_to_open_db(const char *filename, struct memblock *mem, GError **error)
+static int try_to_xslt_open_csv(const char *filename, struct memblock *mem, char **error)
+{
+	char *buf;
+
+	if (readfile(filename, mem) < 0) {
+		if (error) {
+			int len = strlen(translate("gettextFromC","Failed to read '%s'")) + strlen(filename);
+			*error = malloc(len);
+			snprintf(*error, len, translate("gettextFromC","Failed to read '%s'"), filename);
+		}
+
+		return 1;
+	}
+
+	/* Surround the CSV file content with XML tags to enable XSLT
+	 * parsing
+	 */
+	buf = realloc(mem->buffer, mem->size + strlen("<csv></csv>"));
+	if (buf != NULL) {
+		memmove(buf + 5, buf, mem->size);
+		memcpy(buf, "<csv>", 5);
+		memcpy(buf + mem->size + 5, "</csv>", 7);
+		mem->size += strlen("<csv></csv>");
+		mem->buffer = buf;
+	} else {
+		/* we can atleast try to strdup a error... */
+		*error = strdup("realloc failed in __func__\n");
+		free(mem->buffer);
+		return 1;
+	}
+
+	return 0;
+}
+
+static int try_to_open_db(const char *filename, struct memblock *mem, char **error)
 {
 	return parse_dm4_buffer(filename, mem->buffer, mem->size, &dive_table, error);
 }
@@ -202,7 +237,7 @@ static int try_to_open_csv(const char *filename, struct memblock *mem, enum csv_
 		struct sample *sample;
 
 		errno = 0;
-		val = g_ascii_strtod(p,&end);
+		val = strtod(p,&end); // FIXME == localization issue
 		if (end == p)
 			break;
 		if (errno)
@@ -223,19 +258,19 @@ static int try_to_open_csv(const char *filename, struct memblock *mem, enum csv_
 	return 1;
 }
 
-static int open_by_filename(const char *filename, const char *fmt, struct memblock *mem, GError **error)
+static int open_by_filename(const char *filename, const char *fmt, struct memblock *mem, char **error)
 {
-	/* Suunto Dive Manager files: SDE */
-	if (!strcasecmp(fmt, "SDE"))
+	/* Suunto Dive Manager files: SDE, ZIP; divelogs.de files: DLD */
+	if (!strcasecmp(fmt, "SDE") || !strcasecmp(fmt, "ZIP") || !strcasecmp(fmt, "DLD"))
 		return try_to_open_zip(filename, mem, error);
 
-	/* divelogs.de files: DLD */
-	if (!strcasecmp(fmt, "DLD"))
-		return try_to_open_zip(filename, mem, error);
-
-	/* Give a shot for plain ZIP */
-	if (!strcasecmp(fmt, "ZIP"))
-		return try_to_open_zip(filename, mem, error);
+	/* CSV files */
+	if (!strcasecmp(fmt, "CSV")) {
+		int len = strlen(translate("gettextFromC","Failed to read '%s'. Use import for CSV files.")) + strlen(filename);
+		*error = malloc(len);
+		snprintf(*error, len, translate("gettextFromC","Failed to read '%s'. Use import for CSV files."), filename);
+		return 1;
+	}
 
 #if ONCE_COCHRAN_IS_SUPPORTED
 	/* Truly nasty intentionally obfuscated Cochran Anal software */
@@ -254,16 +289,19 @@ static int open_by_filename(const char *filename, const char *fmt, struct memblo
 	return 0;
 }
 
-static void parse_file_buffer(const char *filename, struct memblock *mem, GError **error)
+static void parse_file_buffer(const char *filename, struct memblock *mem, char **error)
 {
 	char *fmt = strrchr(filename, '.');
 	if (fmt && open_by_filename(filename, fmt+1, mem, error))
 		return;
 
-	parse_xml_buffer(filename, mem->buffer, mem->size, &dive_table, error);
+	if (!mem->size || !mem->buffer)
+		return;
+
+	parse_xml_buffer(filename, mem->buffer, mem->size, &dive_table, NULL, error);
 }
 
-void parse_file(const char *filename, GError **error)
+void parse_file(const char *filename, char **error)
 {
 	struct memblock mem;
 	char *fmt;
@@ -273,12 +311,10 @@ void parse_file(const char *filename, GError **error)
 		if (prefs.default_filename && ! strcmp(filename, prefs.default_filename))
 			return;
 
-		g_warning(_("Failed to read '%s'.\n"), filename);
 		if (error) {
-			*error = g_error_new(g_quark_from_string("subsurface"),
-					     DIVE_ERROR_PARSE,
-					     _("Failed to read '%s'"),
-					     filename);
+			int len = strlen(translate("gettextFromC","Failed to read '%s'")) + strlen(filename);
+			*error = malloc(len);
+			snprintf(*error, len, translate("gettextFromC","Failed to read '%s'"), filename);
 		}
 
 		return;
@@ -295,3 +331,74 @@ void parse_file(const char *filename, GError **error)
 	parse_file_buffer(filename, &mem, error);
 	free(mem.buffer);
 }
+
+#define MAXCOLDIGITS 3
+#define MAXCOLS 100
+void parse_csv_file(const char *filename, int timef, int depthf, int tempf, int po2f, int cnsf, int stopdepthf, int sepidx, char **error)
+{
+	struct memblock mem;
+	int pnr=0;
+	char *params[19];
+	char timebuf[MAXCOLDIGITS];
+	char depthbuf[MAXCOLDIGITS];
+	char tempbuf[MAXCOLDIGITS];
+	char po2buf[MAXCOLDIGITS];
+	char cnsbuf[MAXCOLDIGITS];
+	char stopdepthbuf[MAXCOLDIGITS];
+	char separator_index[MAXCOLDIGITS];
+	time_t now;
+	struct tm *timep;
+	char curdate[9];
+	char curtime[6];
+
+	if (timef >= MAXCOLS || depthf >= MAXCOLS || tempf >= MAXCOLS || po2f >= MAXCOLS || cnsf >= MAXCOLS || stopdepthf >= MAXCOLS ) {
+		int len = strlen(translate("gettextFromC", "Maximum number of supported columns on CSV import is %d")) + MAXCOLDIGITS;
+		*error = malloc(len);
+		snprintf(*error, len, translate("gettextFromC", "Maximum number of supported columns on CSV import is %d"), MAXCOLS);
+
+		return;
+	}
+	snprintf(timebuf, MAXCOLDIGITS, "%d", timef);
+	snprintf(depthbuf, MAXCOLDIGITS, "%d", depthf);
+	snprintf(tempbuf, MAXCOLDIGITS, "%d", tempf);
+	snprintf(po2buf, MAXCOLDIGITS, "%d", po2f);
+	snprintf(cnsbuf, MAXCOLDIGITS, "%d", cnsf);
+	snprintf(stopdepthbuf, MAXCOLDIGITS, "%d", stopdepthf);
+	snprintf(separator_index, MAXCOLDIGITS, "%d", sepidx);
+	time(&now);
+	timep = localtime(&now);
+	strftime(curdate, sizeof(curdate), "%Y%m%d", timep);
+
+	/* As the parameter is numeric, we need to ensure that the leading zero
+	* is not discarded during the transform, thus prepend time with 1 */
+	strftime(curtime, sizeof(curtime), "1%H%M", timep);
+
+	params[pnr++] = "timeField";
+	params[pnr++] = timebuf;
+	params[pnr++] = "depthField";
+	params[pnr++] = depthbuf;
+	params[pnr++] = "tempField";
+	params[pnr++] = tempbuf;
+	params[pnr++] = "po2Field";
+	params[pnr++] = po2buf;
+	params[pnr++] = "cnsField";
+	params[pnr++] = cnsbuf;
+	params[pnr++] = "stopdepthField";
+	params[pnr++] = stopdepthbuf;
+	params[pnr++] = "date";
+	params[pnr++] = curdate;
+	params[pnr++] = "time";
+	params[pnr++] = curtime;
+	params[pnr++] = "separatorIndex";
+	params[pnr++] = separator_index;
+	params[pnr++] = NULL;
+
+	if (filename == NULL)
+		return;
+
+	if (try_to_xslt_open_csv(filename, &mem, error))
+		return;
+
+	parse_xml_buffer(filename, mem.buffer, mem.size, &dive_table, (const char **)params, error);
+	free(mem.buffer);
+}
