@@ -4,20 +4,21 @@
  *
  * (c) Dirk Hohndel 2013
  */
-#include <libintl.h>
-#include <glib/gi18n.h>
+#include <assert.h>
 #include <unistd.h>
 #include <ctype.h>
+#include <string.h>
 #include "dive.h"
 #include "divelist.h"
 #include "planner.h"
+#include "gettext.h"
 
-int decostoplevels[] = { 0, 3000, 6000, 9000, 12000, 15000, 18000, 21000, 24000, 27000,
-		     30000, 33000, 36000, 39000, 42000, 45000, 48000, 51000, 54000, 57000,
-		     60000, 63000, 66000, 69000, 72000, 75000, 78000, 81000, 84000, 87000,
-		     90000, 100000, 110000, 120000, 130000, 140000, 150000, 160000, 170000,
-		     180000, 190000, 200000, 220000, 240000, 260000, 280000, 300000,
-		     320000, 340000, 360000, 380000
+unsigned int decostoplevels[] = { 0, 3000, 6000, 9000, 12000, 15000, 18000, 21000, 24000, 27000,
+				  30000, 33000, 36000, 39000, 42000, 45000, 48000, 51000, 54000, 57000,
+				  60000, 63000, 66000, 69000, 72000, 75000, 78000, 81000, 84000, 87000,
+				  90000, 100000, 110000, 120000, 130000, 140000, 150000, 160000, 170000,
+				  180000, 190000, 200000, 220000, 240000, 260000, 280000, 300000,
+				  320000, 340000, 360000, 380000
 };
 double plangflow, plangfhigh;
 char *disclaimer;
@@ -47,7 +48,7 @@ void dump_plan(struct diveplan *diveplan)
 }
 #endif
 
-void set_last_stop(gboolean last_stop_6m)
+void set_last_stop(bool last_stop_6m)
 {
 	if (last_stop_6m == TRUE)
 		decostoplevels[1] = 6000;
@@ -57,6 +58,8 @@ void set_last_stop(gboolean last_stop_6m)
 
 void get_gas_from_events(struct divecomputer *dc, int time, int *o2, int *he)
 {
+	// we don't modify the values passed in if nothing is found
+	// so don't call with uninitialized o2/he !
 	struct event *event = dc->events;
 	while (event && event->time.seconds <= time) {
 		if (!strcmp(event->name, "gaschange")) {
@@ -69,12 +72,12 @@ void get_gas_from_events(struct divecomputer *dc, int time, int *o2, int *he)
 
 /* simple helper function to compare two permille values with
  * (rounded) percent granularity */
-static inline gboolean match_percent(int a, int b)
+static inline bool match_percent(int a, int b)
 {
 	return (a + 5) / 10 == (b + 5) / 10;
 }
 
-static int get_gasidx(struct dive *dive, int o2, int he)
+int get_gasidx(struct dive *dive, int o2, int he)
 {
 	int gasidx = -1;
 
@@ -91,15 +94,15 @@ static int get_gasidx(struct dive *dive, int o2, int he)
 void get_gas_string(int o2, int he, char *text, int len)
 {
 	if (is_air(o2, he))
-		snprintf(text, len, _("air"));
+		snprintf(text, len, "%s", translate("gettextFromC","air"));
 	else if (he == 0)
-		snprintf(text, len, _("EAN%d"), (o2 + 5) / 10);
+		snprintf(text, len, translate("gettextFromC","EAN%d"), (o2 + 5) / 10);
 	else
 		snprintf(text, len, "(%d/%d)",  (o2 + 5) / 10, (he + 5) / 10);
 }
 
 /* returns the tissue tolerance at the end of this (partial) dive */
-double tissue_at_end(struct dive *dive, char **cached_datap, char **error_string_p)
+double tissue_at_end(struct dive *dive, char **cached_datap, const char **error_string_p)
 {
 	struct divecomputer *dc;
 	struct sample *sample, *psample;
@@ -128,7 +131,7 @@ double tissue_at_end(struct dive *dive, char **cached_datap, char **error_string
 		t1 = sample->time.seconds;
 		get_gas_from_events(&dive->dc, t0, &o2, &he);
 		if ((gasidx = get_gasidx(dive, o2, he)) == -1) {
-			snprintf(buf, sizeof(buf),_("Can't find gas %d/%d"), (o2 + 5) / 10, (he + 5) / 10);
+			snprintf(buf, sizeof(buf),translate("gettextFromC","Can't find gas %d/%d"), (o2 + 5) / 10, (he + 5) / 10);
 			*error_string_p = buf;
 			gasidx = 0;
 		}
@@ -146,7 +149,7 @@ double tissue_at_end(struct dive *dive, char **cached_datap, char **error_string
 }
 
 /* how many seconds until we can ascend to the next stop? */
-int time_at_last_depth(struct dive *dive, int o2, int he, int next_stop, char **cached_data_p, char **error_string_p)
+static int time_at_last_depth(struct dive *dive, int o2, int he, unsigned int next_stop, char **cached_data_p, const char **error_string_p)
 {
 	int depth, gasidx;
 	double surface_pressure, tissue_tolerance;
@@ -168,7 +171,33 @@ int time_at_last_depth(struct dive *dive, int o2, int he, int next_stop, char **
 	return wait;
 }
 
-int add_gas(struct dive *dive, int o2, int he)
+void fill_default_cylinder(cylinder_t *cyl)
+{
+	const char *cyl_name = prefs.default_cylinder != NULL ? prefs.default_cylinder : "AL80";
+	struct tank_info_t *ti = tank_info;
+	struct tank_info_t *al80 = NULL;
+
+	while (ti->name != NULL) {
+		if (strcmp(ti->name, cyl_name) == 0)
+			break;
+		if (strcmp(ti->name, "AL80") == 0)
+			al80 = ti;
+		ti++;
+	}
+	if (ti->name == NULL)
+		ti = al80;
+	cyl->type.description = strdup(ti->name);
+	if (ti->ml) {
+		cyl->type.size.mliter = ti->ml;
+		cyl->type.workingpressure.mbar = ti->bar * 1000;
+	} else {
+		cyl->type.workingpressure.mbar = psi_to_mbar(ti->psi);
+		if (ti->psi)
+			cyl->type.size.mliter = cuft_to_l(ti->cuft) * 1000 / bar_to_atm(psi_to_bar(ti->psi));
+	}
+}
+
+static int add_gas(struct dive *dive, int o2, int he)
 {
 	int i;
 	struct gasmix *mix;
@@ -185,15 +214,14 @@ int add_gas(struct dive *dive, int o2, int he)
 	if (i == MAX_CYLINDERS) {
 		return -1;
 	}
+	/* let's make it our default cylinder */
+	fill_default_cylinder(cyl);
 	mix->o2.permille = o2;
 	mix->he.permille = he;
-	/* since air is stored as 0/0 we need to set a name or an air cylinder
-	 * would be seen as unset (by cylinder_nodata()) */
-	cyl->type.description = strdup(_("Cylinder for planning"));
 	return i;
 }
 
-struct dive *create_dive_from_plan(struct diveplan *diveplan, char **error_string)
+struct dive *create_dive_from_plan(struct diveplan *diveplan, const char **error_string)
 {
 	struct dive *dive;
 	struct divedatapoint *dp;
@@ -214,7 +242,7 @@ struct dive *create_dive_from_plan(struct diveplan *diveplan, char **error_strin
 	dive->when = diveplan->when;
 	dive->dc.surface_pressure.mbar = diveplan->surface_pressure;
 	dc = &dive->dc;
-	dc->model = strdup(_("Simulated Dive"));
+	dc->model = "planned dive"; /* do not translate here ! */
 	dp = diveplan->dp;
 
 	/* let's start with the gas given on the first segment */
@@ -259,11 +287,10 @@ struct dive *create_dive_from_plan(struct diveplan *diveplan, char **error_strin
 		if (o2 != oldo2 || he != oldhe) {
 			int plano2 = (o2 + 5) / 10 * 10;
 			int planhe = (he + 5) / 10 * 10;
-			int value;
-			if (add_gas(dive, plano2, planhe) < 0)
+			int idx;
+	    if ((idx = add_gas(dive, plano2, planhe)) < 0)
 				goto gas_error_exit;
-			value = (plano2 / 10) | ((planhe / 10) << 16);
-			add_event(dc, lasttime, 25, 0, value, "gaschange"); // SAMPLE_EVENT_GASCHANGE2
+			add_gas_switch_event(dive, dc, lasttime, idx);
 			oldo2 = o2; oldhe = he;
 		}
 		/* Create sample */
@@ -291,7 +318,7 @@ struct dive *create_dive_from_plan(struct diveplan *diveplan, char **error_strin
 
 gas_error_exit:
 	free(dive);
-	*error_string = _("Too many gas mixes");
+	*error_string = translate("gettextFromC","Too many gas mixes");
 	return NULL;
 }
 
@@ -336,45 +363,6 @@ struct divedatapoint *get_nth_dp(struct diveplan *diveplan, int idx)
 	return dp;
 }
 
-/* return -1 to warn about potentially very long calculation */
-int add_duration_to_nth_dp(struct diveplan *diveplan, int idx, int duration, gboolean is_rel)
-{
-	struct divedatapoint *pdp, *dp = get_nth_dp(diveplan, idx);
-	if (idx > 0) {
-		pdp = get_nth_dp(diveplan, idx - 1);
-		if (duration && (is_rel || duration <= pdp->time))
-			duration += pdp->time;
-	}
-	dp->time = duration;
-	if (duration > 180 * 60)
-		return -1;
-	return 0;
-}
-
-/* this function is ONLY called from the dialog callback - so it
- * marks this entry as 'entered'.
- * Do NOT call from other parts of the planning code without changing
- * that logic */
-void add_depth_to_nth_dp(struct diveplan *diveplan, int idx, int depth)
-{
-	struct divedatapoint *dp = get_nth_dp(diveplan, idx);
-	dp->depth = depth;
-	dp->entered = TRUE;
-}
-
-void add_gas_to_nth_dp(struct diveplan *diveplan, int idx, int o2, int he)
-{
-	struct divedatapoint *dp = get_nth_dp(diveplan, idx);
-	dp->o2 = o2;
-	dp->he = he;
-}
-
-void add_po2_to_nth_dp(struct diveplan *diveplan, int idx, int po2)
-{
-	struct divedatapoint *dp = get_nth_dp(diveplan, idx);
-	dp->po2 = po2;
-}
-
 void add_to_end_of_diveplan(struct diveplan *diveplan, struct divedatapoint *dp)
 {
 	struct divedatapoint **lastdp = &diveplan->dp;
@@ -387,18 +375,19 @@ void add_to_end_of_diveplan(struct diveplan *diveplan, struct divedatapoint *dp)
 		lastdp = &(*lastdp)->next;
 	}
 	*lastdp = dp;
-	if (ldp)
+	if (ldp && dp->time != 0)
 		dp->time += lasttime;
 }
 
-void plan_add_segment(struct diveplan *diveplan, int duration, int depth, int o2, int he, int po2)
+struct divedatapoint * plan_add_segment(struct diveplan *diveplan, int duration, int depth, int o2, int he, int po2)
 {
 	struct divedatapoint *dp = create_dp(duration, depth, o2, he, po2);
 	add_to_end_of_diveplan(diveplan, dp);
+	return(dp);
 }
 
 struct gaschanges {
-	int depth;
+	unsigned int depth;
 	int gasidx;
 };
 
@@ -421,6 +410,7 @@ static struct gaschanges *analyze_gaslist(struct diveplan *diveplan, struct dive
 				i++;
 			}
 			gaschanges[i].depth = dp->depth;
+			gaschanges[i].gasidx = -1;
 			do {
 				if (dive->cylinder[j].gasmix.o2.permille == dp->o2 &&
 				    dive->cylinder[j].gasmix.he.permille == dp->he) {
@@ -429,6 +419,7 @@ static struct gaschanges *analyze_gaslist(struct diveplan *diveplan, struct dive
 				}
 				j++;
 			} while (j < MAX_CYLINDERS);
+			assert(gaschanges[i].gasidx != -1);
 		}
 		dp = dp->next;
 	}
@@ -443,15 +434,15 @@ static struct gaschanges *analyze_gaslist(struct diveplan *diveplan, struct dive
 }
 
 /* sort all the stops into one ordered list */
-static int *sort_stops(int *dstops, int dnr, struct gaschanges *gstops, int gnr)
+static unsigned int *sort_stops(unsigned int *dstops, int dnr, struct gaschanges *gstops, int gnr)
 {
 	int i, gi, di;
 	int total = dnr + gnr;
-	int *stoplevels = malloc(total * sizeof(int));
+	unsigned int *stoplevels = malloc(total * sizeof(unsigned int));
 
 	/* no gaschanges */
 	if (gnr == 0) {
-		memcpy(stoplevels, dstops, dnr * sizeof(int));
+		memcpy(stoplevels, dstops, dnr * sizeof(unsigned int));
 		return stoplevels;
 	}
 	i = total - 1;
@@ -495,6 +486,7 @@ static int *sort_stops(int *dstops, int dnr, struct gaschanges *gstops, int gnr)
 	return stoplevels;
 }
 
+#if DO_WE_WANT_THIS_IN_QT
 static void add_plan_to_notes(struct diveplan *diveplan, struct dive *dive)
 {
 	char buffer[20000];
@@ -506,7 +498,7 @@ static void add_plan_to_notes(struct diveplan *diveplan, struct dive *dive)
 	if (!dp)
 		return;
 
-	snprintf(buffer, sizeof(buffer), _("%s\nSubsurface dive plan\nbased on GFlow = %.0f and GFhigh = %.0f\n\n"),
+	snprintf(buffer, sizeof(buffer), translate("gettextFromC","%s\nSubsurface dive plan\nbased on GFlow = %.0f and GFhigh = %.0f\n\n"),
 					disclaimer, plangflow * 100, plangfhigh * 100);
 	/* we start with gas 0, then check if that was changed */
 	o2 = dive->cylinder[0].gasmix.o2.permille;
@@ -547,7 +539,7 @@ static void add_plan_to_notes(struct diveplan *diveplan, struct dive *dive)
 		if (dp->depth != lastdepth) {
 			used = diveplan->bottomsac / 1000.0 * depth_to_mbar((dp->depth + lastdepth) / 2, dive) *
 						(dp->time - lasttime) / 60;
-			snprintf(buffer + len, sizeof(buffer) - len, _("Transition to %.*f %s in %d:%02d min - runtime %d:%02u on %s\n"),
+			snprintf(buffer + len, sizeof(buffer) - len, translate("gettextFromC","Transition to %.*f %s in %d:%02d min - runtime %d:%02u on %s\n"),
 							decimals, depthvalue, depth_unit,
 							FRACTION(dp->time - lasttime, 60),
 							FRACTION(dp->time, 60),
@@ -556,7 +548,7 @@ static void add_plan_to_notes(struct diveplan *diveplan, struct dive *dive)
 			/* we use deco SAC rate during the calculated deco stops, bottom SAC rate everywhere else */
 			int sac = dp->entered ? diveplan->bottomsac : diveplan->decosac;
 			used = sac / 1000.0 * depth_to_mbar(dp->depth, dive) * (dp->time - lasttime) / 60;
-			snprintf(buffer + len, sizeof(buffer) - len, _("Stay at %.*f %s for %d:%02d min - runtime %d:%02u on %s\n"),
+			snprintf(buffer + len, sizeof(buffer) - len, translate("gettextFromC","Stay at %.*f %s for %d:%02d min - runtime %d:%02u on %s\n"),
 							decimals, depthvalue, depth_unit,
 							FRACTION(dp->time - lasttime, 60),
 							FRACTION(dp->time, 60),
@@ -567,7 +559,7 @@ static void add_plan_to_notes(struct diveplan *diveplan, struct dive *dive)
 		get_gas_string(newo2, newhe, gas, sizeof(gas));
 		if (o2 != newo2 || he != newhe) {
 			len = strlen(buffer);
-			snprintf(buffer + len, sizeof(buffer) - len, _("Switch gas to %s\n"), gas);
+			snprintf(buffer + len, sizeof(buffer) - len, translate("gettextFromC","Switch gas to %s\n"), gas);
 		}
 		o2 = newo2;
 		he = newhe;
@@ -575,7 +567,7 @@ static void add_plan_to_notes(struct diveplan *diveplan, struct dive *dive)
 		lastdepth = dp->depth;
 	} while ((dp = dp->next) != NULL);
 	len = strlen(buffer);
-	snprintf(buffer + len, sizeof(buffer) - len, _("Gas consumption:\n"));
+	snprintf(buffer + len, sizeof(buffer) - len, translate("gettextFromC","Gas consumption:\n"));
 	for (gasidx = 0; gasidx < MAX_CYLINDERS; gasidx++) {
 		double volume;
 		const char *unit;
@@ -586,24 +578,25 @@ static void add_plan_to_notes(struct diveplan *diveplan, struct dive *dive)
 		volume = get_volume_units(consumption[gasidx], NULL, &unit);
 		get_gas_string(dive->cylinder[gasidx].gasmix.o2.permille,
 				dive->cylinder[gasidx].gasmix.he.permille, gas, sizeof(gas));
-		snprintf(buffer + len, sizeof(buffer) - len, _("%.0f%s of %s\n"), volume, unit, gas);
+		snprintf(buffer + len, sizeof(buffer) - len, translate("gettextFromC","%.0f%s of %s\n"), volume, unit, gas);
 	}
 	dive->notes = strdup(buffer);
 }
+#endif
 
-void plan(struct diveplan *diveplan, char **cached_datap, struct dive **divep, char **error_string_p)
+void plan(struct diveplan *diveplan, char **cached_datap, struct dive **divep, bool add_deco, const char **error_string_p)
 {
 	struct dive *dive;
 	struct sample *sample;
 	int wait_time, o2, he, po2;
-	int ceiling, depth, transitiontime;
-	int stopidx, gi;
+	int transitiontime, gi;
+	unsigned int stopidx, depth, ceiling;
 	double tissue_tolerance;
-	struct gaschanges *gaschanges;
+	struct gaschanges *gaschanges = NULL;
 	int gaschangenr;
-	int *stoplevels;
+	unsigned int *stoplevels = NULL;
 
-	set_gf(plangflow, plangfhigh);
+	set_gf(diveplan->gflow, diveplan->gfhigh, default_prefs.gf_low_at_maxdepth);
 	if (!diveplan->surface_pressure)
 		diveplan->surface_pressure = SURFACE_PRESSURE;
 	if (*divep)
@@ -620,6 +613,19 @@ void plan(struct diveplan *diveplan, char **cached_datap, struct dive **divep, c
 	get_gas_from_events(&dive->dc, sample->time.seconds, &o2, &he);
 	po2 = dive->dc.sample[dive->dc.samples - 1].po2;
 	depth = dive->dc.sample[dive->dc.samples - 1].depth.mm;
+
+	/* if all we wanted was the dive just get us back to the surface */
+	if (!add_deco) {
+		transitiontime = depth / 75; /* this still needs to be made configurable */
+		plan_add_segment(diveplan, transitiontime, 0, o2, he, po2);
+		/* re-create the dive */
+		delete_single_dive(dive_table.nr - 1);
+		*divep = dive = create_dive_from_plan(diveplan, error_string_p);
+		if (dive)
+			record_dive(dive);
+		return;
+	}
+
 	tissue_tolerance = tissue_at_end(dive, cached_datap, error_string_p);
 	ceiling = deco_allowed_depth(tissue_tolerance, diveplan->surface_pressure / 1000.0, dive, 1);
 #if DEBUG_PLAN & 4
@@ -628,10 +634,13 @@ void plan(struct diveplan *diveplan, char **cached_datap, struct dive **divep, c
 #endif
 	if (depth < ceiling) /* that's not good... */
 		depth = ceiling;
+	if (depth == 0 && ceiling == 0) /* we are done here */
+		goto done;
 	for (stopidx = 0; stopidx < sizeof(decostoplevels) / sizeof(int); stopidx++)
-		if (decostoplevels[stopidx] >= depth)
+		if (decostoplevels[stopidx] >= ceiling)
 			break;
-	stopidx--;
+	if (stopidx > 0)
+		stopidx--;
 
 	/* so now we know the first decostop level above us
 	 * NOTE, this could be the surface or a long list of potential stops
@@ -645,6 +654,7 @@ void plan(struct diveplan *diveplan, char **cached_datap, struct dive **divep, c
 	gi = gaschangenr - 1;
 	stopidx += gaschangenr;
 	if (depth > stoplevels[stopidx]) {
+		/* right now all the transitions are at 30ft/min - this needs to be configurable */
 		transitiontime = (depth - stoplevels[stopidx]) / 150;
 #if DEBUG_PLAN & 2
 		printf("transitiontime %d:%02d to depth %5.2lfm\n", FRACTION(transitiontime, 60), stoplevels[stopidx] / 1000.0);
@@ -685,6 +695,7 @@ void plan(struct diveplan *diveplan, char **cached_datap, struct dive **divep, c
 #endif
 		if (wait_time)
 			plan_add_segment(diveplan, wait_time, stoplevels[stopidx], o2, he, po2);
+		/* right now all the transitions are at 30ft/min - this needs to be configurable */
 		transitiontime = (stoplevels[stopidx] - stoplevels[stopidx - 1]) / 150;
 #if DEBUG_PLAN & 2
 		printf("transitiontime %d:%02d to depth %5.2lfm\n", FRACTION(transitiontime, 60), stoplevels[stopidx - 1] / 1000.0);
@@ -698,10 +709,13 @@ void plan(struct diveplan *diveplan, char **cached_datap, struct dive **divep, c
 		record_dive(dive);
 		stopidx--;
 	}
+
+done:
+
+#if DO_WE_WANT_THIS_IN_QT
 	add_plan_to_notes(diveplan, dive);
-	/* now make the dive visible in the dive list */
-	report_dives(FALSE, FALSE);
-	show_and_select_dive(dive);
+#endif
+
 error_exit:
 	free(stoplevels);
 	free(gaschanges);
@@ -735,30 +749,6 @@ static int get_tenths(const char *begin, const char **endp)
 	return value;
 }
 
-static int get_thousandths(const char *begin, const char **endp)
-{
-	char *end;
-	int value = strtol(begin, &end, 10);
-
-	if (begin == end)
-		return -1;
-	value *= 1000;
-
-	/* now deal with up to three more digits after decimal point */
-	if (*end == '.') {
-		int factor = 100;
-		do {
-			++end;
-			if (!isdigit(*end))
-				break;
-			value += (*end - '0') * factor;
-			factor /= 10;
-		} while (factor);
-	}
-	*endp = end;
-	return value;
-}
-
 static int get_permille(const char *begin, const char **end)
 {
 	int value = get_tenths(begin, end);
@@ -777,15 +767,15 @@ int validate_gas(const char *text, int *o2_p, int *he_p)
 	if (!text)
 		return 0;
 
-	while (g_ascii_isspace(*text))
+	while (isspace(*text))
 		text++;
 
 	if (!*text)
 		return 0;
 
-	if (!strcasecmp(text, _("air"))) {
-		o2 = O2_IN_AIR; he = 0; text += strlen(_("air"));
-	} else if (!strncasecmp(text, _("ean"), 3)) {
+	if (!strcasecmp(text, translate("gettextFromC","air"))) {
+		o2 = O2_IN_AIR; he = 0; text += strlen(translate("gettextFromC","air"));
+	} else if (!strncasecmp(text, translate("gettextFromC","ean"), 3)) {
 		o2 = get_permille(text+3, &text); he = 0;
 	} else {
 		o2 = get_permille(text, &text); he = 0;
@@ -794,7 +784,7 @@ int validate_gas(const char *text, int *o2_p, int *he_p)
 	}
 
 	/* We don't want any extra crud */
-	while (g_ascii_isspace(*text))
+	while (isspace(*text))
 		text++;
 	if (*text)
 		return 0;
@@ -809,112 +799,6 @@ int validate_gas(const char *text, int *o2_p, int *he_p)
 	return 1;
 }
 
-int validate_time(const char *text, int *sec_p, int *rel_p)
-{
-	int min, sec, rel;
-	char *end;
-
-	if (!text)
-		return 0;
-
-	while (g_ascii_isspace(*text))
-		text++;
-
-	rel = 1;
-	if (*text == '+') {
-		rel = 1;
-		text++;
-		while (g_ascii_isspace(*text))
-			text++;
-	} else if (*text == '@') {
-		rel = 0;
-		text++;
-		while (g_ascii_isspace(*text))
-			text++;
-	}
-
-	min = strtol(text, &end, 10);
-	if (text == end)
-		return 0;
-
-	if (min < 0 || min > 1000)
-		return 0;
-
-	/* Ok, minutes look ok */
-	text = end;
-	sec = 0;
-	if (*text == ':') {
-		text++;
-		sec = strtol(text, &end, 10);
-		if (end != text+2)
-			return 0;
-		if (sec < 0)
-			return 0;
-		text = end;
-		if (*text == ':') {
-			if (sec >= 60)
-				return 0;
-			min = min*60 + sec;
-			text++;
-			sec = strtol(text, &end, 10);
-			if (end != text+2)
-				return 0;
-			if (sec < 0)
-				return 0;
-			text = end;
-		}
-	}
-
-	/* Maybe we should accept 'min' at the end? */
-	if (g_ascii_isspace(*text))
-		text++;
-	if (*text)
-		return 0;
-
-	*sec_p = min*60 + sec;
-	*rel_p = rel;
-	return 1;
-}
-
-int validate_depth(const char *text, int *mm_p)
-{
-	int depth, imperial;
-
-	if (!text)
-		return 0;
-
-	depth = get_tenths(text, &text);
-	if (depth < 0)
-		return 0;
-
-	while (g_ascii_isspace(*text))
-		text++;
-
-	imperial = get_units()->length == FEET;
-	if (*text == 'm') {
-		imperial = 0;
-		text++;
-	} else if (!strcasecmp(text, _("ft"))) {
-		imperial = 1;
-		text += 2;
-	}
-	while (g_ascii_isspace(*text))
-		text++;
-	if (*text)
-		return 0;
-
-	if (imperial) {
-		depth = feet_to_mm(depth / 10.0);
-	} else {
-		depth *= 100;
-	}
-	*mm_p = depth;
-	/* we don't support extreme depths */
-	if (depth > 400000)
-		return 0;
-	return 1;
-}
-
 int validate_po2(const char *text, int *mbar_po2)
 {
 	int po2;
@@ -926,10 +810,10 @@ int validate_po2(const char *text, int *mbar_po2)
 	if (po2 < 0)
 		return 0;
 
-	while (g_ascii_isspace(*text))
+	while (isspace(*text))
 		text++;
 
-	while (g_ascii_isspace(*text))
+	while (isspace(*text))
 		text++;
 	if (*text)
 		return 0;
@@ -938,77 +822,3 @@ int validate_po2(const char *text, int *mbar_po2)
 	return 1;
 }
 
-int validate_volume(const char *text, int *sac)
-{
-	int volume, imperial;
-
-	if (!text)
-		return 0;
-
-	volume = get_thousandths(text, &text);
-	if (volume < 0)
-		return 0;
-
-	while (g_ascii_isspace(*text))
-		text++;
-
-	imperial = get_units()->volume == CUFT;
-	if (*text == 'l') {
-		imperial = 0;
-		text++;
-	} else if (!strncasecmp(text, _("cuft"), 4)) {
-		imperial = 1;
-		text += 4;
-	}
-	while (g_ascii_isspace(*text) || *text == '/')
-		text++;
-	if (!strncasecmp(text, _("min"), 3))
-		text += 3;
-	while (g_ascii_isspace(*text))
-		text++;
-	if (*text)
-		return 0;
-	if (imperial)
-		volume = cuft_to_l(volume) + 0.5;	/* correct for mcuft -> ml */
-	*sac = volume;
-	return 1;
-}
-
-struct diveplan diveplan = {};
-char *cache_data = NULL;
-struct dive *planned_dive = NULL;
-
-/* make a copy of the diveplan so far and display the corresponding dive */
-void show_planned_dive(char **error_string_p)
-{
-	struct diveplan tempplan;
-	struct divedatapoint *dp, **dpp;
-
-	memcpy(&tempplan, &diveplan, sizeof(struct diveplan));
-	dpp = &tempplan.dp;
-	dp = diveplan.dp;
-	while (dp && *dpp) {
-		*dpp = malloc(sizeof(struct divedatapoint));
-		memcpy(*dpp, dp, sizeof(struct divedatapoint));
-		dp = dp->next;
-		dpp = &(*dpp)->next;
-	}
-#if DEBUG_PLAN & 1
-	printf("in show_planned_dive:\n");
-	dump_plan(&tempplan);
-#endif
-	plan(&tempplan, &cache_data, &planned_dive, error_string_p);
-	free_dps(tempplan.dp);
-}
-
-/* Subsurface follows the lead of most divecomputers to use times
- * without timezone - so all times are implicitly assumed to be
- * local time of the dive location; so in order to give the current
- * time in that way we actually need to add the timezone offset */
-timestamp_t current_time_notz(void)
-{
-	time_t now = time(NULL);
-	struct tm *local = localtime(&now);
-	return utc_mktime(local);
-}
-
