@@ -203,7 +203,7 @@ void DiveListView::selectDive(int i, bool scrollto, bool toggle)
 	QModelIndex idx = match.first();
 	flags = toggle ? QItemSelectionModel::Toggle : QItemSelectionModel::Select;
 	flags |= QItemSelectionModel::Rows;
-	selectionModel()->select(idx, flags);
+	selectionModel()->setCurrentIndex(idx, flags);
 	if(idx.parent().isValid()){
 		setAnimated(false);
 		expand(idx.parent());
@@ -581,7 +581,7 @@ void DiveListView::addToTripAbove()
 {
 	int idx, delta = (currentOrder == Qt::AscendingOrder) ? -1 : +1;
 	dive_trip_t *trip = NULL;
-	struct dive *pd;
+	struct dive *pd = NULL;
 	struct dive *d = (struct dive *) contextMenuIndex.data(DiveTripModel::DIVE_ROLE).value<void*>();
 	if (!d) // shouldn't happen as we only are setting up this action if this is a dive
 		return;
@@ -765,7 +765,7 @@ void DiveListView::saveSelectedDivesAs()
 	settings.setValue("LastDir",fileInfo.dir().path());
 	settings.endGroup();
 
-	QByteArray bt = fileName.toLocal8Bit();
+	QByteArray bt = QFile::encodeName(fileName);
 	save_dives_logic(bt.data(), TRUE);
 }
 
