@@ -31,6 +31,7 @@ MainTab::MainTab(QWidget *parent) : QTabWidget(parent),
 				    editMode(NONE)
 {
 	ui.setupUi(this);
+	ui.tagWidget->setFocusPolicy(Qt::StrongFocus); // Don't get focus by 'Wheel'
 	ui.cylinders->setModel(cylindersModel);
 	ui.weights->setModel(weightModel);
 	closeMessage();
@@ -90,6 +91,11 @@ MainTab::MainTab(QWidget *parent) : QTabWidget(parent),
 	completers.location = new QCompleter(LocationCompletionModel::instance(), ui.location);
 	completers.suit = new QCompleter(SuitCompletionModel::instance(), ui.suit);
 	completers.tags = new QCompleter(TagCompletionModel::instance(), ui.tagWidget);
+	completers.buddy->setCaseSensitivity(Qt::CaseInsensitive);
+	completers.divemaster->setCaseSensitivity(Qt::CaseInsensitive);
+	completers.location->setCaseSensitivity(Qt::CaseInsensitive);
+	completers.suit->setCaseSensitivity(Qt::CaseInsensitive);
+	completers.buddy->setCaseSensitivity(Qt::CaseInsensitive);
 	completers.tags->setCaseSensitivity(Qt::CaseInsensitive);
 	ui.buddy->setCompleter(completers.buddy);
 	ui.divemaster->setCompleter(completers.divemaster);
@@ -177,8 +183,7 @@ void MainTab::toggleTriggeredColumn()
 
 void MainTab::addDiveStarted()
 {
-	enableEdition();
-	editMode = ADD;
+	enableEdition(ADD);
 }
 
 void MainTab::addMessageAction(QAction* action)
@@ -220,9 +225,16 @@ void MainTab::displayMessage(QString str)
 
 void MainTab::enableEdition(EditMode newEditMode)
 {
-	if (selected_dive < 0 || editMode != NONE)
+	if (current_dive == NULL || editMode != NONE)
 		return;
-
+	if ((newEditMode == DIVE || newEditMode == NONE) &&
+	    current_dive->dc.model &&
+	    strcmp(current_dive->dc.model, "manually added dive") == 0) {
+		// editCurrentDive will call enableEdition with newEditMode == MANUALLY_ADDED_DIVE
+		// so exit this function here after editCurrentDive() returns
+		mainWindow()->editCurrentDive();
+		return;
+	}
 	mainWindow()->dive_list()->setEnabled(false);
 	mainWindow()->globe()->prepareForGetDiveCoordinates();
 	// We may be editing one or more dives here. backup everything.
@@ -281,20 +293,18 @@ void MainTab::enableEdition(EditMode newEditMode)
 
 bool MainTab::eventFilter(QObject* object, QEvent* event)
 {
-	// we want to prevent the user from accidentally enabling editMode:
-	// for the tagWidget we ignore FocusIn - that's both a click and starting the scroll wheel
-	// this means a click by itself won't start edit mode - but typing something will
-	if (isEnabled() && editMode == NONE && object->objectName() == "tagWidget" &&
-	    event->type() == QEvent::FocusIn)
-		return true;
+	if (!isEnabled())
+		return false;
+
+	if (editMode != NONE)
+		return false;
 	// for the dateTimeEdit widget we need to ignore Wheel events as well (as long as we aren't editing)
-	if (isEnabled() && editMode == NONE && object->objectName() == "dateTimeEdit" &&
+	if (object->objectName() == "dateTimeEdit" &&
 	    (event->type() == QEvent::FocusIn || event->type() == QEvent::Wheel))
 		return true;
 	// MouseButtonPress in any widget (not all will ever get this), KeyPress in the dateTimeEdit,
 	// FocusIn for the starWidgets or RequestSoftwareInputPanel for tagWidget start the editing
-	if (isEnabled() && editMode == NONE &&
-	    (event->type() == QEvent::MouseButtonPress) ||
+	if ((event->type() == QEvent::MouseButtonPress) ||
 	    (event->type() == QEvent::KeyPress && object == ui.dateTimeEdit) ||
 	    (event->type() == QEvent::FocusIn && (object == ui.rating || object == ui.visibility)) ||
 	    (event->type() == QEvent::RequestSoftwareInputPanel && object == ui.tagWidget)) {
@@ -358,7 +368,7 @@ bool MainTab::isEditing()
 void MainTab::updateDiveInfo(int dive)
 {
 	// don't execute this while adding a dive
-	if (editMode == ADD)
+	if (editMode == ADD || editMode == MANUALLY_ADDED_DIVE)
 		return;
 	if (!isEnabled() && dive != -1)
 		setEnabled(true);
@@ -373,6 +383,7 @@ void MainTab::updateDiveInfo(int dive)
 	temperature_t temp;
 	struct dive *prevd;
 	struct dive *d = get_dive(dive);
+	char buf[1024];
 
 	process_selected_dives();
 	process_all_dives(d, &prevd);
@@ -414,6 +425,8 @@ void MainTab::updateDiveInfo(int dive)
 			ui.location->setText(currentTrip->location);
 			ui.NotesLabel->setText(tr("Trip Notes"));
 			ui.notes->setText(currentTrip->notes);
+			clearEquipment();
+			ui.equipmentTab->setEnabled(false);
 		} else {
 			setTabText(0, tr("Dive Notes"));
 			// make all the fields visible writeable
@@ -441,6 +454,12 @@ void MainTab::updateDiveInfo(int dive)
 			// reset labels in case we last displayed trip notes
 			ui.LocationLabel->setText(tr("Location"));
 			ui.NotesLabel->setText(tr("Notes"));
+			ui.equipmentTab->setEnabled(true);
+			multiEditEquipmentPlaceholder = *d;
+			cylindersModel->setDive(&multiEditEquipmentPlaceholder);
+			weightModel->setDive(&multiEditEquipmentPlaceholder);
+			taglist_get_tagstring(d->tag_list, buf, 1024);
+			ui.tagWidget->setText(QString(buf));
 		}
 		ui.maximumDepthText->setText(get_depth_string(d->maxdepth, TRUE));
 		ui.averageDepthText->setText(get_depth_string(d->meandepth, TRUE));
@@ -513,15 +532,6 @@ void MainTab::updateDiveInfo(int dive)
 		ui.timeLimits->setAverage(get_time_string(seconds, 0));
 		ui.timeLimits->setMaximum(get_time_string(stats_selection.longest_time.seconds, 0));
 		ui.timeLimits->setMinimum(get_time_string(stats_selection.shortest_time.seconds, 0));
-
-
-		char buf[1024];
-		taglist_get_tagstring(d->tag_list, buf, 1024);
-		ui.tagWidget->setText(QString(buf));
-
-		multiEditEquipmentPlaceholder = *d;
-		cylindersModel->setDive(&multiEditEquipmentPlaceholder);
-		weightModel->setDive(&multiEditEquipmentPlaceholder);
 	} else {
 		/* clear the fields */
 		clearInfo();
@@ -565,6 +575,7 @@ void MainTab::acceptChanges()
 	tabBar()->setTabIcon(0, QIcon()); // Notes
 	tabBar()->setTabIcon(1, QIcon()); // Equipment
 	hideMessage();
+	ui.equipmentTab->setEnabled(true);
 	/* now figure out if things have changed */
 	if (mainWindow() && mainWindow()->dive_list()->selectedTrips().count() == 1) {
 		if (notesBackup[NULL].notes != ui.notes->toPlainText() ||
@@ -669,6 +680,7 @@ void MainTab::acceptChanges()
 		mainWindow()->dive_list()->restoreSelection();
 	}
 	mainWindow()->dive_list()->verticalScrollBar()->setSliderPosition(scrolledBy);
+	mainWindow()->dive_list()->setFocus();
 }
 
 void MainTab::resetPallete()
@@ -687,12 +699,12 @@ void MainTab::resetPallete()
 }
 
 #define EDIT_TEXT2(what, text) \
-	textByteArray = text.toLocal8Bit(); \
+	textByteArray = text.toUtf8(); \
 	free(what);\
 	what = strdup(textByteArray.data());
 
 #define EDIT_TEXT(what, text) \
-	QByteArray textByteArray = text.toLocal8Bit(); \
+	QByteArray textByteArray = text.toUtf8(); \
 	free(what);\
 	what = strdup(textByteArray.data());
 
@@ -792,6 +804,7 @@ void MainTab::rejectChanges()
 		mainWindow()->refreshDisplay(false);
 		DivePlannerPointsModel::instance()->setPlanMode(DivePlannerPointsModel::NOTHING);
 	}
+	mainWindow()->dive_list()->setFocus();
 }
 #undef EDIT_TEXT2
 
@@ -812,7 +825,12 @@ void MainTab::rejectChanges()
 
 void markChangedWidget(QWidget *w){
 	QPalette p;
-	p.setBrush(QPalette::Base, QColor(Qt::yellow).lighter());
+	qreal h, s, l, a;
+	qApp->palette().color(QPalette::Text).getHslF(&h, &s, &l, &a);
+	p.setBrush(QPalette::Base, ( l <= 0.3 ) ? QColor(Qt::yellow).lighter()
+				  :( l <= 0.6 ) ? QColor(Qt::yellow).light()
+				  :/* else */     QColor(Qt::yellow).darker(300)
+				  );
 	w->setPalette(p);
 }
 
@@ -909,7 +927,7 @@ void MainTab::on_notes_textChanged()
 		// we are editing a trip
 		dive_trip_t *currentTrip = *mainWindow()->dive_list()->selectedTrips().begin();
 		EDIT_TEXT(currentTrip->notes, ui.notes->toPlainText());
-	} else if (editMode == DIVE || editMode == ADD) {
+	} else if (editMode == DIVE || editMode == ADD || editMode == MANUALLY_ADDED_DIVE) {
 		EDIT_SELECTED_DIVES( EDIT_TEXT(mydive->notes, ui.notes->toPlainText()) );
 	}
 	markChangedWidget(ui.notes);
@@ -919,11 +937,12 @@ void MainTab::on_notes_textChanged()
 
 void MainTab::on_coordinates_textChanged(const QString& text)
 {
-	bool gpsChanged = FALSE;
-	EDIT_SELECTED_DIVES(gpsChanged |= gpsHasChanged(mydive, NULL, text));
+	bool gpsChanged = false;
+	bool parsed = false;
+	EDIT_SELECTED_DIVES(gpsChanged |= gpsHasChanged(mydive, current_dive, text, &parsed));
 	if (gpsChanged) {
 		markChangedWidget(ui.coordinates);
-	} else {
+	} else if (!parsed) {
 		QPalette p;
 		p.setBrush(QPalette::Base, QColor(Qt::red).lighter());
 		ui.coordinates->setPalette(p);
@@ -976,8 +995,8 @@ QString MainTab::printGPSCoords(int lat, int lon)
 	ilatmin = (lat % 1000000) * 60;
 	ilonmin = (lon % 1000000) * 60;
 	result.sprintf("%s%u%s %2d.%05d\' , %s%u%s %2d.%05d\'",
-		       lath.toLocal8Bit().data(), latdeg, UTF8_DEGREE, ilatmin / 1000000, (ilatmin % 1000000) / 10,
-		       lonh.toLocal8Bit().data(), londeg, UTF8_DEGREE, ilonmin / 1000000, (ilonmin % 1000000) / 10);
+		       lath.toUtf8().data(), latdeg, UTF8_DEGREE, ilatmin / 1000000, (ilatmin % 1000000) / 10,
+		       lonh.toUtf8().data(), londeg, UTF8_DEGREE, ilonmin / 1000000, (ilonmin % 1000000) / 10);
 	return result;
 }
 
