@@ -161,6 +161,17 @@ void ProfileGraphicsView::contextMenuEvent(QContextMenuEvent* event)
 		m.addAction(action);
 		break;
 	}
+	bool some_hidden = false;
+	for (int i = 0; i < evn_used; i++) {
+		if (ev_namelist[i].plot_ev == false) {
+			some_hidden = true;
+			break;
+		}
+	}
+	if (some_hidden) {
+		action = m.addAction(tr("Unhide all events"), this, SLOT(unhideEvents()));
+		action->setData(event->globalPos());
+	}
 	m.exec(event->globalPos());
 }
 
@@ -213,6 +224,14 @@ void ProfileGraphicsView::hideEvents()
 	}
 }
 
+void ProfileGraphicsView::unhideEvents()
+{
+	for (int i = 0; i < evn_used; i++) {
+		ev_namelist[i].plot_ev = true;
+	}
+	plot(current_dive, TRUE);
+}
+
 void ProfileGraphicsView::removeEvent()
 {
 	QAction *action = qobject_cast<QAction*>(sender());
@@ -476,6 +495,8 @@ void ProfileGraphicsView::plot(struct dive *d, bool forceRedraw)
 
 	if (rulerEnabled && !printMode)
 		add_ruler();
+
+	gc.rightx = get_maxtime(&gc.pi);
 }
 
 void ProfileGraphicsView::plot_depth_scale()
@@ -486,11 +507,7 @@ void ProfileGraphicsView::plot_depth_scale()
 	/* Depth markers: every 30 ft or 10 m*/
 	maxdepth = get_maxdepth(&gc.pi);
 	gc.topy = 0; gc.bottomy = maxdepth;
-
-	switch (prefs.units.length) {
-		case units::METERS: marker = 10000; break;
-		case units::FEET: marker = 9144; break;	/* 30 ft */
-	}
+	marker = M_OR_FT(10,30);
 
 	/* don't write depth labels all the way to the bottom as
 	 * there may be other graphs below the depth plot (like
@@ -1136,14 +1153,7 @@ void ProfileGraphicsView::plot_depth_profile()
 	/* Depth markers: every 30 ft or 10 m*/
 	gc.leftx = 0; gc.rightx = 1.0;
 	gc.topy = 0; gc.bottomy = maxdepth;
-	switch (prefs.units.length) {
-	case units::METERS:
-		marker = 10000;
-		break;
-	case units::FEET:
-		marker = 9144;
-		break;	/* 30 ft */
-	}
+	marker = M_OR_FT(10,30);
 	maxline = qMax(gc.pi.maxdepth + marker, maxdepth * 2 / 3);
 
 	c = getColor(DEPTH_GRID);
@@ -1167,6 +1177,12 @@ void ProfileGraphicsView::plot_depth_profile()
 		pen.setColor(c);
 		item->setPen(pen);
 		scene()->addItem(item);
+
+		struct text_render_options tro = {DEPTH_TEXT_SIZE, MEAN_DEPTH, LEFT, TOP};
+		QString depthLabel = get_depth_string(gc.pi.meandepth, true, true);
+		plot_text(&tro, QPointF(gc.leftx, gc.pi.meandepth), depthLabel, item);
+		tro.hpos = RIGHT;
+		plot_text(&tro, QPointF(gc.pi.entry[gc.pi.nr - 1].sec, gc.pi.meandepth), depthLabel, item);
 	}
 
 #if 0
@@ -1630,7 +1646,7 @@ QColor EventItem::getColor(const color_indice_t i)
 
 EventItem::EventItem(struct event *ev, QGraphicsItem* parent, bool grayscale): QGraphicsPixmapItem(parent), ev(ev), isGrayscale(grayscale)
 {
-	if(ev->name && strcmp(ev->name, "bookmark") == 0) {
+	if(ev->name && (strcmp(ev->name, "bookmark") == 0 || strcmp(ev->name, "heading") == 0)) {
 		setPixmap( QPixmap(QString(":flag")).scaled(20, 20, Qt::KeepAspectRatio, Qt::SmoothTransformation));
 	} else {
 		setPixmap( QPixmap(QString(":warning")).scaled(20, 20, Qt::KeepAspectRatio, Qt::SmoothTransformation));
