@@ -4,6 +4,14 @@
  * core logic functions called from statistics UI
  * common types and variables
  */
+
+#ifndef STATISTICS_H
+#define STATISTICS_H
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 typedef struct {
 	int period;
 	duration_t total_time;
@@ -22,12 +30,25 @@ typedef struct {
 	unsigned int combined_count;
 	unsigned int selection_size;
 	unsigned int total_sac_time;
+	bool is_year;
+	bool is_trip;
+	char *location;
 } stats_t;
 extern stats_t stats_selection;
 extern stats_t *stats_yearly;
 extern stats_t *stats_monthly;
+extern stats_t *stats_by_trip;
 
 extern char *get_time_string(int seconds, int maxdays);
 extern char *get_minutes(int seconds);
 extern void process_all_dives(struct dive *dive, struct dive **prev_dive);
 extern void get_selected_dives_text(char *buffer, int size);
+extern void get_gas_used(struct dive *dive, volume_t gases[MAX_CYLINDERS]);
+extern char *get_gaslist(struct dive *dive);
+extern void process_selected_dives(void);
+
+#ifdef __cplusplus
+}
+#endif
+
+#endif
